#include <iostream>
#include <iterator>
#include <vector>
#include <fstream>
#include "FDS/rt_api.h"

using namespace std;

namespace {
  string g_service_name = "FDS1";
  int g_max_wait_time_in_ms = 100;
}

// Callback function for all messages, simply prints out the message contents
void on_message(int tag, const FDS::RT_Message *msg, const FDS::RT_Record *rec,
                void *closure)
{
  RT_LOG_INFO("Message Received" << endl << *msg);

  if (msg->is_closed()) {
    FDS::RT_Consumer *con = reinterpret_cast<FDS::RT_Consumer *>(closure);
    con->cancel(tag);
  }
}

int main()
{
  ifstream infile("MultiUserRequests.txt");
  string line;
  vector<FDS::RT_Consumer*> cons;
  if (!infile.good()) {
    RT_LOG_ERROR("Could not open file");
    return -1;
  }
  while (getline(infile, line)) {
    // Parse each line of the input file.  Each line is as follows:
    // (for basic authentication)
    // username-serial:password@host TICKER-ISO TICKER-ISO TICKER-ISO ...
    // (for otp authentication)
    // username-serial host full-path-of-otp-data-file TICKER-ISO TICKER-ISO TICKER-ISO ...
    // This loop will make a consumer for each line of the file,
    // and make requests under that consumer for every TICKER-ISO in that line.
    stringstream ss(line);
    istream_iterator<string> begin(ss);
    istream_iterator<string> end;
    vector<string> v_req(begin, end);
    if (v_req.size() < 2) {
      continue;
    }

    FDS::RT_Consumer *con = new FDS::RT_Consumer();
    if (v_req[0].find("@") != string::npos) {
      con->set_connection_info(v_req[0].c_str());
      const      FDS::rt_errno err = con->connect();
      if (err) {
        RT_LOG_ERROR("Error connecting to server with basic authentication for " << v_req[0] << " e:" << err);
        continue;
      }

      for (size_t i = 1; i < v_req.size(); i++) {
        FDS::RT_Request req(g_service_name.c_str(), v_req[i].c_str());
        int tag = 0;
        con->request(req, on_message, con, &tag);
      }
    }
    else { // otp
      if (v_req.size() < 4) { // should contain atleast 4 string in format (user host otp_file symbol)
        RT_LOG_ERROR("otp authentication should contain atleast 4 attributes.");
        continue;
      }
      string user = v_req[0];
      string host = v_req[1];
      string otp_path = v_req[2];
      size_t sep_loc = otp_path.find_last_of("/\\");
      string path = "";
      string device_id = "";

      if (sep_loc != string::npos)
      {
        path = otp_path.substr(0, sep_loc);
        device_id = otp_path.substr(sep_loc + 1);
      }
      else
      {
        device_id = otp_path;
      }

      // Removing .data extension from the key-counter data file
      size_t ext_loc = device_id.find(".data\0");
      if (ext_loc != string::npos)
      {
        device_id = device_id.substr(0, ext_loc);
      }

      con->set_connection_info(host.c_str(), user.c_str(), device_id.c_str(), NULL, NULL, path.c_str(), false);
      const      FDS::rt_errno err = con->connect();
      if (err) {
        RT_LOG_ERROR("Error connecting to server with otp authentication for user:" << user << " e:" << err);
        continue;
      }

      for (size_t i = 3; i < v_req.size(); i++) {
        FDS::RT_Request req(g_service_name.c_str(), v_req[i].c_str());
        int tag = 0;
        con->request(req, on_message, con, &tag);
      }
    }
    cons.push_back(con);
  }

  // Since this program is single-threaded, it's not possible to dispatch
  // every consumer simultaneously.  Instead, it will loop through the
  // consumers, dispatching on each one individually, returning after the
  // specified wait time has elapsed with no messages received.
  while (true) {
    for (size_t i = 0; i < cons.size(); i++) {
      cons[i]->dispatch(g_max_wait_time_in_ms);
    }
  }

  return 0;
}