import java.io.*;
import java.util.zip.*;
import java.net.HttpURLConnection;
import java.net.URL;
import javax.net.ssl.HttpsURLConnection;
import javax.xml.parsers.*;
import org.xml.sax.*;

public class MyXMLParser implements ContentHandler {

private static String my_url           = "https://datadirect-beta.factset.com/services/DFSnapshot";
private static String my_authorization = "Basic AaBbCcDdEeFfGgHhIi1234=="; // Base-64 encoding of "user:password"

private static HttpsURLConnection create_http_get_request(StringBuilder symbols, 
                                                          StringBuilder fields) throws IOException
{
    URL url = new URL(my_url + "?fmt=x&serv=FDS1&symbols=" + symbols + "&fields=" + fields);
    HttpsURLConnection conn = (HttpsURLConnection)url.openConnection();
    conn.setDoOutput(true);
    conn.setRequestProperty("Accept-Encoding", "deflate, gzip");
    conn.setRequestProperty("Authorization", my_authorization);
    return conn;
}

private static HttpsURLConnection create_http_post_request(StringBuilder symbols, 
                                                           StringBuilder fields) throws IOException
{
    URL url = new URL(my_url);
    HttpsURLConnection conn = (HttpsURLConnection)url.openConnection();
    conn.setDoOutput(true);
    conn.setRequestProperty("Accept-Encoding", "deflate, gzip");
    conn.setRequestProperty("Authorization", my_authorization);
    conn.setRequestMethod("POST");
    
    // Fill in the Request Stream with the POST data
    String postData = "fmt=x&serv=FDS1&symbols=" + symbols + "&fields=" + fields;    	
    OutputStream wr = conn.getOutputStream();
    wr.write(postData.getBytes());
    wr.flush();
    return conn;
}

private static StringBuilder join(String delim, String [] array)
{
    StringBuilder sb = new StringBuilder();
    for ( int i = 0 ; i < array.length ; ++i ) {
        if (i != 0) sb.append(delim);
        sb.append(array[i]);
    }
    return sb;
}

public static void main(String[] args)
{
    try {
        // setup the request
        String[] symbol_list = { "FDS-USA", "IBM-USA", "TWX-USA" };
        String[] field_list  = { "ASK_1", "BID_1", "LAST_1" };
        
        StringBuilder symbols = join(",", symbol_list);
        StringBuilder fields  = join(",", field_list);
        
        // comment out this line and uncomment the next to use HTTP post
        HttpsURLConnection conn = create_http_get_request(symbols, fields);
        // HttpsURLConnection conn = create_http_post_request(symbols, fields);
        
        // getInputStream will implicitly connect and get the response
        InputStream inputStream = conn.getInputStream();
        
        // uncompress the response
        if (conn.getContentEncoding().equals("deflate"))
            inputStream = new InflaterInputStream(inputStream, new Inflater(true));
        else if (conn.getContentEncoding().equals("gzip"))
            inputStream = new GZIPInputStream(inputStream);
        
        XMLReader reader = SAXParserFactory.newInstance().newSAXParser().getXMLReader();
        reader.setContentHandler(new MyXMLParser());
        reader.parse(new InputSource(inputStream));
    } catch (IOException e) {
            // TODO: Deal with IOException
    } catch (SAXException e) {
            // TODO: Deal with SAXException
    } catch (ParserConfigurationException e) {
            // TODO: Deal with ParserConfigurationException
    }	
}
		
public void startElement(String uri, String localName, String qName, Attributes atts)
{
    if (qName.equals("Error"))
        System.out.println("Error code: " + atts.getValue("code")
                           + ", description: " + atts.getValue("description"));
    else if (qName.equals("Record")) {
        currentReqSym = atts.getValue("req_sym");
        currentKey = atts.getValue("key");
        currentStale = atts.getValue("stale");
    }
    else if (qName.equals("Field"))
        System.out.println("req_sym: " + currentReqSym
                           + ", key: " + currentKey + ", stale: " + currentStale
                           + ", id: " + atts.getValue("id")
                           + ", name: " + atts.getValue("name")
                           + " = " + atts.getValue("value"));
}

private String currentReqSym, currentKey, currentStale;

public void characters(char [] ch, int start, int length) {}
public void endDocument() {}
public void endElement(String uri, String localName, String qName) {}
public void endPrefixMapping(String prefix) {}
public void ignorableWhitespace(char[] ch, int start, int length) {}
public void processingInstruction(String target, String data) {}
public void setDocumentLocator(Locator locator) {}
public void skippedEntity(String name) {}
public void startDocument() {}
public void startPrefixMapping(String prefix, String uri) {}

} // class MyXMLParser
